@@ -70,9 +70,13 @@
 /* and the real names of the random devices */
 #undef NAME_OF_DEV_RANDOM
 #undef NAME_OF_DEV_URANDOM
+/* Linux has an ioctl */
+#undef HAVE_DEV_RANDOM_IOCTL
+
 
 #undef USE_DYNAMIC_LINKING
 #undef HAVE_DL_DLOPEN
+#undef HAVE_DL_SHL_LOAD
 #undef HAVE_DLD_DLD_LINK
 
 #undef USE_SHM_COPROCESSING
