@@ -63,6 +63,7 @@ enum cmd_and_opt_values { aNull = 0,
     oUser	  = 'u',
     oVerbose	  = 'v',
     oCompress	  = 'z',
+    oNotation	  = 'N',
     oBatch	  = 500,
     aClearsign,
     aStore,
@@ -104,6 +105,7 @@ enum cmd_and_opt_values { aNull = 0,
 
     oTextmode,
     oFingerprint,
+    oWithFingerprint,
     oAnswerYes,
     oAnswerNo,
     oKeyring,
@@ -115,11 +117,13 @@ enum cmd_and_opt_values { aNull = 0,
     oStatusFD,
     oNoComment,
     oNoVersion,
+    oEmitVersion,
     oCompletesNeeded,
     oMarginalsNeeded,
     oMaxCertDepth,
     oLoadExtension,
     oRFC1991,
+    oOpenPGP,
     oCipherAlgo,
     oDigestAlgo,
     oCompressAlgo,
@@ -131,10 +135,12 @@ enum cmd_and_opt_values { aNull = 0,
     oNoArmor,
     oNoDefKeyring,
     oNoGreeting,
+    oNoTTY,
     oNoOptions,
     oNoBatch,
     oHomedir,
     oWithColons,
+    oWithKeyData,
     oSkipVerify,
     oCompressKeys,
     oCompressSigs,
@@ -142,7 +148,10 @@ enum cmd_and_opt_values { aNull = 0,
     oEmuChecksumBug,
     oRunAsShmCP,
     oSetFilename,
+    oSetPolicyURL,
+    oUseEmbeddedFilename,
     oComment,
+    oDefaultComment,
     oThrowKeyid,
     oForceV3Sigs,
     oForceMDC,
@@ -153,10 +162,13 @@ enum cmd_and_opt_values { aNull = 0,
     oNotDashEscaped,
     oEscapeFrom,
     oLockOnce,
+    oLockMultiple,
     oKeyServer,
     oEncryptTo,
     oNoEncryptTo,
     oLoggerFD,
+    oUtf8Strings,
+    oNoUtf8Strings,
 aTest };
 
 
@@ -222,6 +234,7 @@ static ARGPARSE_OPTS opts[] = {
     { oOutput, "output",    2, N_("use as output file")},
     { oVerbose, "verbose",   0, N_("verbose") },
     { oQuiet,	"quiet",   0, N_("be somewhat more quiet") },
+    { oNoTTY, "no-tty", 0, N_("don't use the terminal at all") },
     { oForceV3Sigs, "force-v3-sigs", 0, N_("force v3 signatures") },
     { oForceMDC, "force-mdc", 0, N_("always use a MDC for encryption") },
     { oDryRun, "dry-run",   0, N_("do not make any changes") },
@@ -245,6 +258,7 @@ static ARGPARSE_OPTS opts[] = {
     { oMaxCertDepth,	"max-cert-depth", 1, "@" },
     { oLoadExtension, "load-extension" ,2, N_("|FILE|load extension module FILE")},
     { oRFC1991, "rfc1991",   0, N_("emulate the mode described in RFC1991")},
+    { oOpenPGP, "openpgp", 0, N_("set all packet, cipher and digest options to OpenPGP behavior")},
     { oS2KMode, "s2k-mode",  1, N_("|N|use passphrase mode N")},
     { oS2KDigest, "s2k-digest-algo",2,
 		N_("|NAME|use message digest algorithm NAME for passphrases")},
@@ -254,6 +268,7 @@ static ARGPARSE_OPTS opts[] = {
     { oDigestAlgo, "digest-algo", 2 , N_("|NAME|use message digest algorithm NAME")},
     { oCompressAlgo, "compress-algo", 1 , N_("|N|use compress algorithm N")},
     { oThrowKeyid, "throw-keyid", 0, N_("throw keyid field of encrypted packets")},
+    { oNotation,   "notation-data", 2, N_("|NAME=VALUE|use this notation data")},
 
     { 302, NULL, 0, N_("@\nExamples:\n\n"
     " -se -r Bob [file]          sign and encrypt for user Bob\n"
@@ -281,6 +296,7 @@ static ARGPARSE_OPTS opts[] = {
     { oHomedir, "homedir", 2, "@" },   /* defaults to "~/.gnupg" */
     { oNoBatch, "no-batch", 0, "@" },
     { oWithColons, "with-colons", 0, "@"},
+    { oWithKeyData,"with-key-data", 0, "@"},
     { aListKeys, "list-key", 0, "@" }, /* alias */
     { aListSigs, "list-sig", 0, "@" }, /* alias */
     { aCheckKeys, "check-sig",0, "@" }, /* alias */
@@ -291,19 +307,27 @@ static ARGPARSE_OPTS opts[] = {
     { oEmuChecksumBug, "emulate-checksum-bug", 0, "@"},
     { oRunAsShmCP, "run-as-shm-coprocess", 4, "@" },
     { oSetFilename, "set-filename", 2, "@" },
+    { oSetPolicyURL, "set-policy-url", 2, "@" },
     { oComment, "comment", 2, "@" },
-    { oNoVersion, "no-version", 0,   "@"},
+    { oDefaultComment, "default-comment", 0, "@" },
+    { oNoVersion, "no-version", 0, "@"},
+    { oEmitVersion, "emit-version", 0, "@"},
     { oNotDashEscaped, "not-dash-escaped", 0, "@" },
     { oEscapeFrom, "escape-from-lines", 0, "@" },
     { oLockOnce, "lock-once", 0, "@" },
+    { oLockMultiple, "lock-multiple", 0, "@" },
     { oLoggerFD, "logger-fd",1, "@" },
+    { oUseEmbeddedFilename, "use-embedded-filename", 0, "@" },
+    { oUtf8Strings, "utf8-strings", 0, "@" },
+    { oNoUtf8Strings, "no-utf8-strings", 0, "@" },
+    { oWithFingerprint, "with-fingerprint", 0, "@" },
 {0} };
 
 
 
 int g10_errors_seen = 0;
 
-
+static int utf8_strings = 0;
 static int maybe_setuid = 1;
 
 static char *build_list( const char *text,
@@ -312,6 +336,8 @@ static void set_cmd( enum cmd_and_opt_values *ret_cmd,
 			enum cmd_and_opt_values new_cmd );
 static void print_hex( byte *p, size_t n );
 static void print_mds( const char *fname, int algo );
+static void add_notation_data( const char *string );
+static int  check_policy_url( const char *s );
 
 const char *
 strusage( int level )
@@ -416,6 +442,19 @@ wrong_args( const char *text)
     g10_exit(2);
 }
 
+
+static char *
+make_username( const char *string )
+{
+    char *p;
+    if( utf8_strings )
+	p = native_to_utf8( string );
+    else
+	p = m_strdup(string);
+    return p;
+}
+
+
 static void
 set_debug(void)
 {
@@ -468,6 +507,7 @@ main( int argc, char **argv )
     int orig_argc;
     char **orig_argv;
     const char *fname;
+    char *username;
     STRLIST sl, remusr= NULL, locusr=NULL;
     STRLIST nrings=NULL, sec_nrings=NULL;
     armor_filter_context_t afx;
@@ -486,6 +526,7 @@ main( int argc, char **argv )
     char *s2k_cipher_string = NULL;
     char *s2k_digest_string = NULL;
     int pwfd = -1;
+    int with_fpr = 0; /* make an option out of --fingerprint */
   #ifdef USE_SHM_COPROCESSING
     ulong requested_shm_size=0;
   #endif
@@ -503,11 +544,11 @@ main( int argc, char **argv )
     create_dotlock(NULL); /* register locking cleanup */
     i18n_init();
     opt.compress = -1; /* defaults to standard compress level */
-    /* fixme: set the next two to zero and decide where used */
+    /* note: if you change these lines, look at oOpenPGP */
     opt.def_cipher_algo = 0;
     opt.def_digest_algo = 0;
     opt.def_compress_algo = 2;
-    opt.s2k_mode = 1; /* salted */
+    opt.s2k_mode = 3; /* iterated+salted */
     opt.s2k_digest_algo = DIGEST_ALGO_RMD160;
     opt.s2k_cipher_algo = CIPHER_ALGO_BLOWFISH;
     opt.completes_needed = 1;
@@ -548,6 +589,11 @@ main( int argc, char **argv )
 	    opt.shm_coprocess = 1;
 	    requested_shm_size = pargs.r.ret_ulong;
 	}
+	else if ( pargs.r_opt == oStatusFD ) {
+	    /* this is needed to ensure that the status-fd filedescriptor is
+	     * initialized when init_shm_coprocessing() is called */
+	    set_status_fd( pargs.r.ret_int );
+	}
       #endif
     }
 
@@ -640,6 +686,7 @@ main( int argc, char **argv )
 	  case oArmor: opt.armor = 1; opt.no_armor=0; break;
 	  case oOutput: opt.outfile = pargs.r.ret_str; break;
 	  case oQuiet: opt.quiet = 1; break;
+	  case oNoTTY: opt.quiet = 1; tty_no_terminal(1); break;
 	  case oDryRun: opt.dry_run = 1; break;
 	  case oInteractive: opt.interactive = 1; break;
 	  case oVerbose: g10_opt_verbose++;
@@ -654,6 +701,8 @@ main( int argc, char **argv )
 	  case oDebugAll: opt.debug = ~0; break;
 	  case oStatusFD: set_status_fd( pargs.r.ret_int ); break;
 	  case oLoggerFD: log_set_logfile( NULL, pargs.r.ret_int ); break;
+	  case oWithFingerprint:
+		with_fpr=1; /*fall thru*/
 	  case oFingerprint: opt.fingerprint++; break;
 	  case oSecretKeyring: append_to_strlist( &sec_nrings, pargs.r.ret_str); break;
 	  case oOptions:
@@ -672,6 +721,7 @@ main( int argc, char **argv )
 	  case oQuickRandom: quick_random_gen(1); break;
 	  case oNoComment: opt.no_comment=1; break;
 	  case oNoVersion: opt.no_version=1; break;
+	  case oEmitVersion: opt.no_version=0; break;
 	  case oCompletesNeeded: opt.completes_needed = pargs.r.ret_int; break;
 	  case oMarginalsNeeded: opt.marginals_needed = pargs.r.ret_int; break;
 	  case oMaxCertDepth: opt.max_cert_depth = pargs.r.ret_int; break;
@@ -680,6 +730,7 @@ main( int argc, char **argv )
 	  case oNoOptions: break; /* no-options */
 	  case oHomedir: opt.homedir = pargs.r.ret_str; break;
 	  case oNoBatch: opt.batch = 0; break;
+	  case oWithKeyData: opt.with_key_data=1; /* fall thru */
 	  case oWithColons: opt.with_colons=':'; break;
 
 	  case oSkipVerify: opt.skip_verify=1; break;
@@ -696,6 +747,20 @@ main( int argc, char **argv )
 	    opt.no_comment = 1;
 	    opt.escape_from = 1;
 	    break;
+	  case oOpenPGP:
+	    opt.rfc1991 = 0;
+	    opt.escape_from = 0;
+	    opt.force_v3_sigs = 0;
+	    opt.compress_keys = 0;	    /* not mandated  but we do it */
+	    opt.compress_sigs = 0;	    /* ditto. */
+	    opt.not_dash_escaped = 0;
+	    opt.def_cipher_algo = 0;
+	    opt.def_digest_algo = 0;
+	    opt.def_compress_algo = 2;
+	    opt.s2k_mode = 3; /* iterated+salted */
+	    opt.s2k_digest_algo = DIGEST_ALGO_RMD160;
+	    opt.s2k_cipher_algo = CIPHER_ALGO_BLOWFISH;
+	    break;
 	  case oEmuChecksumBug: opt.emulate_bugs |= EMUBUG_GPGCHKSUM; break;
 	  case oCompressSigs: opt.compress_sigs = 1; break;
 	  case oRunAsShmCP:
@@ -706,7 +771,10 @@ main( int argc, char **argv )
 	  #endif
 	    break;
 	  case oSetFilename: opt.set_filename = pargs.r.ret_str; break;
+	  case oSetPolicyURL: opt.set_policy_url = pargs.r.ret_str; break;
+	  case oUseEmbeddedFilename: opt.use_embedded_filename = 1; break;
 	  case oComment: opt.comment_string = pargs.r.ret_str; break;
+	  case oDefaultComment: opt.comment_string = NULL; break;
 	  case oThrowKeyid: opt.throw_keyid = 1; break;
 	  case oForceV3Sigs: opt.force_v3_sigs = 1; break;
 	  case oForceMDC: opt.force_mdc = 1; break;
@@ -716,16 +784,16 @@ main( int argc, char **argv )
 
 	  case oNoEncryptTo: opt.no_encrypt_to = 1; break;
 	  case oEncryptTo: /* store the recipient in the second list */
-	    sl = add_to_strlist( &remusr, pargs.r.ret_str );
+	    sl = add_to_strlist2( &remusr, pargs.r.ret_str, utf8_strings );
 	    sl->flags = 1;
 	    break;
 	  case oRecipient: /* store the recipient */
-	    add_to_strlist( &remusr, pargs.r.ret_str );
+	    add_to_strlist2( &remusr, pargs.r.ret_str, utf8_strings );
 	    break;
 	  case oTextmodeShort: opt.textmode = 2; break;
 	  case oTextmode: opt.textmode=1;  break;
 	  case oUser: /* store the local users */
-	    add_to_strlist( &locusr, pargs.r.ret_str );
+	    add_to_strlist2( &locusr, pargs.r.ret_str, utf8_strings );
 	    break;
 	  case oCompress: opt.compress = pargs.r.ret_int; break;
 	  case oPasswdFD: pwfd = pargs.r.ret_int; break;
@@ -740,7 +808,11 @@ main( int argc, char **argv )
 	  case oNotDashEscaped: opt.not_dash_escaped = 1; break;
 	  case oEscapeFrom: opt.escape_from = 1; break;
 	  case oLockOnce: opt.lock_once = 1; break;
+	  case oLockMultiple: opt.lock_once = 0; break;
 	  case oKeyServer: opt.keyserver_name = pargs.r.ret_str; break;
+	  case oNotation: add_notation_data( pargs.r.ret_str ); break;
+	  case oUtf8Strings: utf8_strings = 1; break;
+	  case oNoUtf8Strings: utf8_strings = 0; break;
 
 	  default : pargs.err = configfp? 1:2; break;
 	}
@@ -797,6 +869,10 @@ main( int argc, char **argv )
 	if( check_digest_algo(opt.s2k_digest_algo) )
 	    log_error(_("selected digest algorithm is invalid\n"));
     }
+    if( opt.set_policy_url ) {
+	if( check_policy_url( opt.set_policy_url ) )
+	    log_error(_("the given policy URL is invalid\n"));
+    }
     if( opt.def_compress_algo < 1 || opt.def_compress_algo > 2 )
 	log_error(_("compress algorithm must be in range %d..%d\n"), 1, 2);
     if( opt.completes_needed < 1 )
@@ -818,7 +894,7 @@ main( int argc, char **argv )
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
-    if( !cmd && opt.fingerprint )
+    if( !cmd && opt.fingerprint && !with_fpr )
 	set_cmd( &cmd, aListKeys);
 
     if( cmd == aKMode || cmd == aKModeC ) { /* kludge to be compatible to pgp */
@@ -974,7 +1050,7 @@ main( int argc, char **argv )
 	if( argc > 1 ) {
 	    sl = NULL;
 	    for( argc--, argv++ ; argc; argc--, argv++ )
-		append_to_strlist( &sl, *argv );
+		append_to_strlist2( &sl, *argv, utf8_strings );
 	    keyedit_menu( fname, locusr, sl );
 	    free_strlist(sl);
 	}
@@ -988,9 +1064,10 @@ main( int argc, char **argv )
       case aDeleteKey:
 	if( argc != 1 )
 	    wrong_args(_("--delete-key username"));
-	/* note: fname is the user id! */
-	if( (rc = delete_key(fname, cmd==aDeleteSecretKey)) )
-	    log_error("%s: delete key failed: %s\n", print_fname_stdin(fname), g10_errstr(rc) );
+	username = make_username( fname );
+	if( (rc = delete_key(username, cmd==aDeleteSecretKey)) )
+	    log_error("%s: delete key failed: %s\n", username, g10_errstr(rc) );
+	m_free(username);
 	break;
 
 
@@ -1005,7 +1082,7 @@ main( int argc, char **argv )
 	secret_key_list( argc, argv );
 	break;
 
-      case aKMode: /* list keyring */
+      case aKMode: /* list keyring -- NOTE: This will be removed soon */
 	if( argc < 2 )	/* -kv [userid] */
 	    public_key_list( (argc && **argv)? 1:0, argv );
 	else if( argc == 2 ) { /* -kv userid keyring */
@@ -1051,7 +1128,7 @@ main( int argc, char **argv )
       case aRecvKeys:
 	sl = NULL;
 	for( ; argc; argc--, argv++ )
-	    add_to_strlist( &sl, *argv );
+	    add_to_strlist2( &sl, *argv, utf8_strings );
 	if( cmd == aSendKeys )
 	    hkp_export( sl );
 	else if( cmd == aRecvKeys )
@@ -1064,7 +1141,7 @@ main( int argc, char **argv )
       case aExportSecret:
 	sl = NULL;
 	for( ; argc; argc--, argv++ )
-	    add_to_strlist( &sl, *argv );
+	    add_to_strlist2( &sl, *argv, utf8_strings );
 	export_seckeys( sl );
 	free_strlist(sl);
 	break;
@@ -1072,7 +1149,9 @@ main( int argc, char **argv )
       case aGenRevoke:
 	if( argc != 1 )
 	    wrong_args("--gen-revoke user-id");
-	gen_revoke( *argv );
+	username =  make_username(*argv);
+	gen_revoke( username );
+	m_free( username );
 	break;
 
       case aDeArmor:
@@ -1195,8 +1274,11 @@ main( int argc, char **argv )
 	if( !argc )
 	    check_trustdb(NULL);
 	else {
-	    for( ; argc; argc--, argv++ )
-		check_trustdb( *argv );
+	    for( ; argc; argc--, argv++ ) {
+		username = make_username( *argv );
+		check_trustdb( username );
+		m_free(username);
+	    }
 	}
 	break;
 
@@ -1209,8 +1291,11 @@ main( int argc, char **argv )
       case aListTrustPath:
 	if( !argc )
 	    wrong_args("--list-trust-path <usernames>");
-	for( ; argc; argc--, argv++ )
-	    list_trust_path( *argv );
+	for( ; argc; argc--, argv++ ) {
+	    username = make_username( *argv );
+	    list_trust_path( username );
+	    m_free(username);
+	}
 	break;
 
       case aExportOwnerTrust:
@@ -1268,8 +1353,10 @@ main( int argc, char **argv )
 void
 g10_exit( int rc )
 {
-    if( opt.debug & DBG_MEMSTAT_VALUE )
+    if( opt.debug & DBG_MEMSTAT_VALUE ) {
 	m_print_stats("on exit");
+	random_dump_stats();
+    }
     if( opt.debug )
 	secmem_dump_stats();
     secmem_term();
@@ -1382,3 +1469,76 @@ print_mds( const char *fname, int algo )
 	fclose(fp);
 }
 
+
+/****************
+ * Check the supplied name,value string and add it to the notation
+ * data to be used for signatures.
+ */
+static void
+add_notation_data( const char *string )
+{
+    const char *s;
+    const char *s2;
+    STRLIST sl;
+    int critical=0;
+    int highbit=0;
+
+    if( *string == '!' ) {
+	critical = 1;
+	string++;
+    }
+    s = string;
+
+    if( !*s || (*s & 0x80) || (!isalpha(*s) && *s != '_') ) {
+	log_error(_("the first character of a notation name "
+		    "must be a letter or an underscore\n") );
+	return;
+    }
+    for(s++; *s != '='; s++ ) {
+	if( !*s || (*s & 0x80) || (!isalnum(*s) && *s != '_' && *s != '.' ) ) {
+	    log_error(_("a notation name must have only letters, "
+			"digits, dots or underscores and end with an '='\n") );
+	    return;
+	}
+    }
+    if( s[-1] == '.' || ((s2=strstr(string, "..")) && s2 < s ) ) {
+	log_error(_("dots in a notation name must be surrounded "
+		    "by other characters\n") );
+	return;
+    }
+    /* we do only support printabe text - therefore we enforce the use
+     * of only printable characters (an empty value is valid) */
+    for( s++; *s ; s++ ) {
+	if( iscntrl(*s) ) {
+	    log_error(_("a notation value must not use "
+			"any control characters\n") );
+	    return;
+	}
+	else if( *s & 0x80 )
+	    highbit = 1;
+    }
+
+    if( highbit )   /* must use UTF8 encoding */
+	sl = add_to_strlist2( &opt.notation_data, string, utf8_strings );
+    else
+	sl = add_to_strlist( &opt.notation_data, string );
+
+    if( critical )
+	sl->flags |= 1;
+}
+
+
+static int
+check_policy_url( const char *s )
+{
+    if( *s == '!' )
+	s++;
+    if( !*s )
+	return -1;
+    for(; *s ; s++ ) {
+	if( (*s & 0x80) || iscntrl(*s) )
+	    return -1;
+    }
+    return 0;
+}
+
