@@ -101,7 +101,10 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
 
 
 { N_("keygen.valid"),
- "Enter the required value"
+ "Enter the required value as shown in the pronpt.\n"
+ "It is possible to enter a ISO date (YYYY-MM-DD) but you won't\n"
+ "get a good error response - instead the system tries to interpret\n"
+ "the given value as an interval."
 },
 
 { N_("keygen.valid.okay"),
@@ -146,10 +149,6 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
 },
 
 
-{ N_("keyedit.cmd"),
- "Please enter \"help\" to see the list of commands."
-},
-
 { N_("keyedit.save.okay"),
  "Answer \"yes\" or \"no\""
 },
@@ -172,6 +171,31 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
  "Answer \"yes\" if it is okay to delete the subkey"
 },
 
+
+{ N_("keyedit.delsig.valid"),
+ "This is a valid signature on the key; you normally don't want\n"
+ "to delete this signature may be important to establish a trust\n"
+ "connection to the key or another key certified by this key."
+},
+{ N_("keyedit.delsig.unknown"),
+ "This signature can't be checked because you don't have the\n"
+ "corresponding key.  You should postpone its deletion until you\n"
+ "know which key was used because this signing key might establish"
+ "a trust connection through another already certified key."
+},
+{ N_("keyedit.delsig.invalid"),
+ "The signature is not valid.  It does make sense to remove it from\n"
+ "your keyring."
+},
+{ N_("keyedit.delsig.selfsig"),
+ "This is a signature which binds the user ID to the key. It is\n"
+ "usually not a good idea to remove such a signature.  Actually\n"
+ "GnuPG might not be able to use this key anymore.  So do this\n"
+ "only if this self-signature is for some reason not valid and\n"
+ "a second one is available."
+},
+
+
 { N_("passphrase.enter"),
  ""
 "Please enter the passhrase; this is a secret sentence \n"
@@ -187,10 +211,17 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
  "Give the name fo the file to which the signature applies"
 },
 
+/* openfile.c (overwrite_filep) */
 { N_("openfile.overwrite.okay"),
  "Answer \"yes\" if it is okay to overwrite the file"
 },
 
+/* openfile.c (ask_outfile_name) */
+{ N_("openfile.askoutname"),
+ "Please enter a new filename. If you just hit RETURN the default\n"
+ "file (which is shown in brackets) will be used."
+},
+
 /* end of list */
 { NULL, NULL } };
 
