@@ -47,6 +47,7 @@ static void show_key_and_fingerprint( KBNODE keyblock );
 static void show_fingerprint( PKT_public_key *pk );
 static int menu_adduid( KBNODE keyblock, KBNODE sec_keyblock );
 static void menu_deluid( KBNODE pub_keyblock, KBNODE sec_keyblock );
+static int  menu_delsig( KBNODE pub_keyblock );
 static void menu_delkey( KBNODE pub_keyblock, KBNODE sec_keyblock );
 static int menu_expire( KBNODE pub_keyblock, KBNODE sec_keyblock );
 static int menu_select_uid( KBNODE keyblock, int idx );
@@ -58,6 +59,7 @@ static int count_selected_uids( KBNODE keyblock );
 static int count_selected_keys( KBNODE keyblock );
 static int menu_revsig( KBNODE keyblock );
 static int menu_revkey( KBNODE pub_keyblock, KBNODE sec_keyblock );
+static int enable_disable_key( KBNODE keyblock, int disable );
 
 #define CONTROL_D ('D' - 'A' + 1)
 
@@ -103,6 +105,68 @@ get_keyblock_byname( KBNODE *keyblock, KBPOS *kbpos, const char *username )
 }
 
 
+/****************
+ * Print information about a signature, chek it and return true
+ * if the signature is okay. NODE must be a signature packet.
+ */
+static int
+print_and_check_one_sig( KBNODE keyblock, KBNODE node,
+			 int *inv_sigs, int *no_key, int *oth_err,
+			int *is_selfsig, int print_without_key )
+{
+    PKT_signature *sig = node->pkt->pkt.signature;
+    int rc, sigrc;
+    int is_rev = sig->sig_class == 0x30;
+
+    switch( (rc = check_key_signature( keyblock, node, is_selfsig)) ) {
+      case 0:
+	node->flag &= ~(NODFLG_BADSIG|NODFLG_NOKEY|NODFLG_SIGERR);
+	sigrc = '!';
+	break;
+      case G10ERR_BAD_SIGN:
+	node->flag = NODFLG_BADSIG;
+	sigrc = '-';
+	if( inv_sigs )
+	    ++*inv_sigs;
+	break;
+      case G10ERR_NO_PUBKEY:
+	node->flag = NODFLG_NOKEY;
+	sigrc = '?';
+	if( no_key )
+	    ++*no_key;
+	break;
+      default:
+	node->flag = NODFLG_SIGERR;
+	sigrc = '%';
+	if( oth_err )
+	    ++*oth_err;
+	break;
+    }
+    if( sigrc != '?' || print_without_key ) {
+	tty_printf("%s%c       %08lX %s   ",
+		is_rev? "rev":"sig",
+		sigrc, sig->keyid[1], datestr_from_sig(sig));
+	if( sigrc == '%' )
+	    tty_printf("[%s] ", g10_errstr(rc) );
+	else if( sigrc == '?' )
+	    ;
+	else if( *is_selfsig ) {
+	    tty_printf( is_rev? _("[revocation]")
+			      : _("[self-signature]") );
+	}
+	else {
+	    size_t n;
+	    char *p = get_user_id( sig->keyid, &n );
+	    tty_print_string( p, n > 40? 40 : n );
+	    m_free(p);
+	}
+	tty_printf("\n");
+    }
+    return (sigrc == '!');
+}
+
+
+
 /****************
  * Check the keysigs and set the flags to indicate errors.
  * Returns true if error found.
@@ -112,7 +176,6 @@ check_all_keysigs( KBNODE keyblock, int only_selected )
 {
     KBNODE kbctx;
     KBNODE node;
-    int rc;
     int inv_sigs = 0;
     int no_key = 0;
     int oth_err = 0;
@@ -140,54 +203,14 @@ check_all_keysigs( KBNODE keyblock, int only_selected )
 	else if( selected && node->pkt->pkttype == PKT_SIGNATURE
 		 && ( (node->pkt->pkt.signature->sig_class&~3) == 0x10
 		     || node->pkt->pkt.signature->sig_class == 0x30 )  ) {
-	    PKT_signature *sig = node->pkt->pkt.signature;
-	    int sigrc, selfsig;
-	    int is_rev = sig->sig_class == 0x30;
+	    int selfsig;
 
-	    switch( (rc = check_key_signature( keyblock, node, &selfsig)) ) {
-	      case 0:
-		node->flag &= ~(NODFLG_BADSIG|NODFLG_NOKEY|NODFLG_SIGERR);
-		sigrc = '!';
-		break;
-	      case G10ERR_BAD_SIGN:
-		node->flag = NODFLG_BADSIG;
-		sigrc = '-';
-		inv_sigs++;
-		break;
-	      case G10ERR_NO_PUBKEY:
-		node->flag = NODFLG_NOKEY;
-		sigrc = '?';
-		no_key++;
-		break;
-	      default:
-		node->flag = NODFLG_SIGERR;
-		sigrc = '%';
-		oth_err++;
-		break;
-	    }
-	    if( sigrc != '?' ) {
-		tty_printf("%s%c       %08lX %s   ",
-			is_rev? "rev":"sig",
-			sigrc, sig->keyid[1], datestr_from_sig(sig));
-		if( sigrc == '%' )
-		    tty_printf("[%s] ", g10_errstr(rc) );
-		else if( sigrc == '?' )
-		    ;
-		else if( selfsig ) {
-		    tty_printf( is_rev? _("[revocation]")
-				      : _("[self-signature]") );
-		    if( sigrc == '!' )
-			has_selfsig = 1;
-		}
-		else {
-		    size_t n;
-		    char *p = get_user_id( sig->keyid, &n );
-		    tty_print_string( p, n > 40? 40 : n );
-		    m_free(p);
-		}
-		tty_printf("\n");
-		/* fixme: Should we update the trustdb here */
+	    if( print_and_check_one_sig( keyblock, node, &inv_sigs,
+					&no_key, &oth_err, &selfsig, 0 ) ) {
+		if( selfsig )
+		    has_selfsig = 1;
 	    }
+	    /* Hmmm: should we update the trustdb here? */
 	}
     }
     if( !has_selfsig )
@@ -428,7 +451,7 @@ change_passphrase( KBNODE keyblock )
 	for(;;) {
 	    s2k->mode = opt.s2k_mode;
 	    s2k->hash_algo = opt.s2k_digest_algo;
-	    dek = passphrase_to_dek( NULL, opt.s2k_cipher_algo, s2k, 2 );
+	    dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2 );
 	    if( !dek ) {
 		tty_printf(_("passphrase not correctly repeated; try again.\n"));
 	    }
@@ -525,9 +548,10 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 {
     enum cmdids { cmdNONE = 0,
 	   cmdQUIT, cmdHELP, cmdFPR, cmdLIST, cmdSELUID, cmdCHECK, cmdSIGN,
-	   cmdLSIGN, cmdREVSIG, cmdREVKEY,
+	   cmdLSIGN, cmdREVSIG, cmdREVKEY, cmdDELSIG,
 	   cmdDEBUG, cmdSAVE, cmdADDUID, cmdDELUID, cmdADDKEY, cmdDELKEY,
 	   cmdTOGGLE, cmdSELKEY, cmdPASSWD, cmdTRUST, cmdPREF, cmdEXPIRE,
+	   cmdENABLEKEY, cmdDISABLEKEY,
 	   cmdNOP };
     static struct { const char *name;
 		    enum cmdids id;
@@ -554,6 +578,7 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	{ N_("deluid")  , cmdDELUID , 0, N_("delete user id") },
 	{ N_("addkey")  , cmdADDKEY , 1, N_("add a secondary key") },
 	{ N_("delkey")  , cmdDELKEY , 0, N_("delete a secondary key") },
+	{ N_("delsig")  , cmdDELSIG , 0, N_("delete signatures") },
 	{ N_("expire")  , cmdEXPIRE , 1, N_("change the expire date") },
 	{ N_("toggle")  , cmdTOGGLE , 1, N_("toggle between secret "
 					    "and public key listing") },
@@ -563,9 +588,11 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	{ N_("trust")   , cmdTRUST , 0, N_("change the ownertrust") },
 	{ N_("revsig")  , cmdREVSIG , 0, N_("revoke signatures") },
 	{ N_("revkey")  , cmdREVKEY , 1, N_("revoke a secondary key") },
+	{ N_("disable") , cmdDISABLEKEY , 0, N_("disable a key") },
+	{ N_("enable")  , cmdENABLEKEY , 0, N_("enable a key") },
 
     { NULL, cmdNONE } };
-    enum cmdids cmd;
+    enum cmdids cmd = 0;
     int rc = 0;
     KBNODE keyblock = NULL;
     KBPOS keyblockpos;
@@ -616,7 +643,7 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
     toggle = 0;
     cur_keyblock = keyblock;
     for(;;) { /* main loop */
-	int i, arg_number;
+	int i, arg_number=0;
 	char *p;
 
 	tty_printf("\n");
@@ -639,7 +666,7 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 		    have_commands = 0;
 	    }
 	    if( !have_commands ) {
-		answer = cpr_get("keyedit.cmd", _("Command> "));
+		answer = cpr_get("", _("Command> "));
 		cpr_kill_prompt();
 	    }
 	    trim_spaces(answer);
@@ -805,6 +832,19 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	    }
 	    break;
 
+	  case cmdDELSIG: {
+		int n1;
+
+		if( !(n1=count_selected_uids(keyblock)) )
+		    tty_printf(_("You must select at least one user id.\n"));
+		else if( menu_delsig( keyblock ) ) {
+		    /* no redisplay here, because it may scroll away some
+		     * status output of delsig */
+		    modified = 1;
+		}
+	    }
+	    break;
+
 	  case cmdADDKEY:
 	    if( generate_subkeypair( keyblock, sec_keyblock ) ) {
 		redisplay = 1;
@@ -896,6 +936,14 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	    }
 	    break;
 
+	  case cmdENABLEKEY:
+	  case cmdDISABLEKEY:
+	    if( enable_disable_key( keyblock, cmd == cmdDISABLEKEY ) ) {
+		redisplay = 1;
+		modified = 1;
+	    }
+	    break;
+
 	  default:
 	    tty_printf("\n");
 	    tty_printf(_("Invalid command  (try \"help\")\n"));
@@ -985,6 +1033,12 @@ show_key_with_all_names( KBNODE keyblock, int only_marked,
 			  expirestr_from_pk(pk) );
 	    if( node->pkt->pkttype == PKT_PUBLIC_KEY ) {
 		tty_printf(" trust: %c/%c", otrust, trust );
+		if( node->pkt->pkttype == PKT_PUBLIC_KEY
+		    && (get_ownertrust( pk->local_id )&TRUST_FLAG_DISABLED)) {
+		    tty_printf("\n*** ");
+		    tty_printf(_("This key has been disabled"));
+		}
+
 		if( with_fpr  ) {
 		    tty_printf("\n");
 		    show_fingerprint( pk );
@@ -1227,6 +1281,70 @@ menu_deluid( KBNODE pub_keyblock, KBNODE sec_keyblock )
 }
 
 
+static int
+menu_delsig( KBNODE pub_keyblock )
+{
+    KBNODE node;
+    PKT_user_id *uid = NULL;
+    int changed=0;
+
+    for( node = pub_keyblock; node; node = node->next ) {
+	if( node->pkt->pkttype == PKT_USER_ID ) {
+	    uid = (node->flag & NODFLG_SELUID)? node->pkt->pkt.user_id : NULL;
+	}
+	else if( uid && node->pkt->pkttype == PKT_SIGNATURE ) {
+	   int okay, valid, selfsig, inv_sig, no_key, other_err;
+
+	    tty_printf("uid  ");
+	    tty_print_string( uid->name, uid->len );
+	    tty_printf("\n");
+
+	   okay = inv_sig = no_key = other_err = 0;
+	    valid = print_and_check_one_sig( pub_keyblock, node,
+					    &inv_sig, &no_key, &other_err,
+					    &selfsig, 1 );
+
+	   if( valid )
+	       okay = cpr_get_answer_yes_no_quit(
+		   "keyedit.delsig.valid",
+		   _("Delete this good signature? (y/N/q)"));
+	   else if( inv_sig || other_err )
+	       okay = cpr_get_answer_yes_no_quit(
+		   "keyedit.delsig.invalid",
+		   _("Delete this invalid signature? (y/N/q)"));
+	   else if( no_key )
+	       okay = cpr_get_answer_yes_no_quit(
+		   "keyedit.delsig.unknown",
+		   _("Delete this unknown signature? (y/N/q)"));
+
+	    if( okay == -1 )
+		break;
+	   if( okay && selfsig && !cpr_get_answer_is_yes(
+			       "keyedit.delsig.selfsig",
+			      _("Really delete this self-signature? (y/N)") ))
+		okay = 0;
+	    if( okay ) {
+		delete_kbnode( node );
+		changed++;
+	    }
+
+	}
+	else if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY )
+	    uid = NULL;
+    }
+
+    if( changed ) {
+	commit_kbnode( &pub_keyblock );
+	tty_printf( changed == 1? _("Deleted %d signature.\n")
+				: _("Deleted %d signatures.\n"), changed );
+    }
+    else
+	tty_printf( _("Nothing deleted.\n") );
+
+    return changed;
+}
+
+
 /****************
  * Remove some of the secondary keys
  */
@@ -1300,9 +1418,9 @@ menu_expire( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	return 0;
     }
     else if( n1 )
-	tty_printf(_("Changing exiration time for a secondary key.\n"));
+	tty_printf(_("Changing expiration time for a secondary key.\n"));
     else {
-	tty_printf(_("Changing exiration time for the primary key.\n"));
+	tty_printf(_("Changing expiration time for the primary key.\n"));
 	mainkey=1;
     }
 
@@ -1757,3 +1875,23 @@ menu_revkey( KBNODE pub_keyblock, KBNODE sec_keyblock )
 }
 
 
+static int
+enable_disable_key( KBNODE keyblock, int disable )
+{
+    ulong lid = find_kbnode( keyblock, PKT_PUBLIC_KEY )
+			    ->pkt->pkt.public_key->local_id;
+    unsigned int trust, newtrust;
+
+    /* Note: Because the keys have beed displayed, we have
+     * ensured that local_id has been set */
+    trust = newtrust = get_ownertrust( lid );
+    newtrust &= ~TRUST_FLAG_DISABLED;
+    if( disable )
+	newtrust |= TRUST_FLAG_DISABLED;
+    if( trust == newtrust )
+	return 0; /* already in that state */
+    if( !update_ownertrust( lid, newtrust ) )
+	return 1;
+    return 0;
+}
+
