@@ -492,6 +492,45 @@ print_fingerprint( PKT_public_key *pk, PKT_secret_key *sk )
     putchar('\n');
 }
 
+static void
+print_notation_data( PKT_signature *sig )
+{
+    size_t n, n1, n2;
+    const byte *p;
+    int seq = 0;
+
+    while( (p = enum_sig_subpkt( sig->hashed_data, SIGSUBPKT_NOTATION,
+				 &n, &seq )) ) {
+	if( n < 8 ) {
+	    log_info(_("WARNING: invalid notation data found\n"));
+	    return;
+	}
+	if( !(*p & 0x80) )
+	    return; /* not human readable */
+	n1 = (p[4] << 8) | p[5];
+	n2 = (p[6] << 8) | p[7];
+	p += 8;
+	if( 8+n1+n2 != n ) {
+	    log_info(_("WARNING: invalid notation data found\n"));
+	    return;
+	}
+	log_info(_("Notation: ") );
+	print_string( log_stream(), p, n1, 0 );
+	putc( '=', log_stream() );
+	print_string( log_stream(), p+n1, n2, 0 );
+	putc( '\n', log_stream() );
+    }
+    if( (p = parse_sig_subpkt( sig->hashed_data, SIGSUBPKT_POLICY, &n ) )) {
+	log_info(_("Policy: ") );
+	print_string( log_stream(), p, n, 0 );
+	putc( '\n', log_stream() );
+    }
+
+    /* Now check wheter the key of this signature has some
+     * notation data */
+
+    /* TODO */
+}
 
 /****************
  * List the certificate in a user friendly way
@@ -935,7 +974,8 @@ check_sig_and_print( CTX c, KBNODE node )
 	    fputs("[?]\"\n", log_stream() );
 	}
 	release_kbnode( keyblock );
-
+	if( !rc )
+	    print_notation_data( sig );
 
 	if( !rc && is_status_enabled() ) {
 	    /* print a status response with the fingerprint */
@@ -1040,10 +1080,23 @@ proc_tree( CTX c, KBNODE node )
 		/* enable a workaround for a pgp2 bug */
 		c->mfx.md2 = md_open( DIGEST_ALGO_MD5, 0 );
 	    }
+	  #if 0
+	    #warning md_start_debug enabled
+	    md_start_debug( c->mfx.md, "det1" );
+	    if( c->mfx.md2 )
+		md_start_debug( c->mfx.md2, "det2" );
+	  #endif
+	    /* Here we have another hack to work around a pgp 2 bug
+	     * It works by not using the textmode for detached signatures;
+	     * this will let the first signazure check (on md) fail
+	     * but the second one (on md2) which adds an extra CR should
+	     * then produce the "correct" hash.  This is very, very ugly
+	     * hack but it may help in some cases (and break others)
+	     */
 	    if( c->sigs_only )
 		rc = hash_datafiles( c->mfx.md, c->mfx.md2,
 				     c->signed_data, c->sigfilename,
-				     sig->sig_class == 0x01 );
+				  c->mfx.md2? 0 :(sig->sig_class == 0x01) );
 	    else
 		rc = ask_for_detached_datafile( &c->mfx,
 					    iobuf_get_fname(c->iobuf));
