@@ -20,6 +20,7 @@
 #ifndef G10_OPTIONS_H
 #define G10_OPTIONS_H
 
+#include <types.h>
 
 #undef ENABLE_COMMENT_PACKETS  /* don't create comment packets */
 
@@ -38,6 +39,7 @@ struct {
     int answer_no;  /* answer no on most questions */
     int check_sigs; /* check key signatures */
     int with_colons;
+    int with_key_data;
     int fingerprint; /* list fingerprints */
     int list_sigs;   /* list signatures */
     int no_armor;
@@ -73,6 +75,9 @@ struct {
     const char *keyserver_name;
     int no_encrypt_to;
     int interactive;
+    STRLIST notation_data;
+    const char *set_policy_url;
+    int use_embedded_filename;
 } opt;
 
 
