@@ -231,7 +231,9 @@ typedef enum {
     SIGSUBPKT_POLICY	   =26, /* policy URL */
     SIGSUBPKT_KEY_FLAGS    =27, /* key flags */
     SIGSUBPKT_SIGNERS_UID  =28, /* signer's user id */
-    SIGSUBPKT_PRIV_ADD_SIG =101 /* signatur is also valid for this uid */
+    SIGSUBPKT_PRIV_ADD_SIG =101,/* signatur is also valid for this uid */
+
+    SIGSUBPKT_FLAG_CRITICAL=128
 } sigsubpkttype_t;
 
 
@@ -264,8 +266,10 @@ int copy_some_packets( IOBUF inp, IOBUF out, ulong stopoff );
 int skip_some_packets( IOBUF inp, unsigned n );
 #endif
 
-const byte *parse_sig_subpkt( const byte *buffer,
-			      sigsubpkttype_t reqtype, size_t *ret_n );
+const byte *enum_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype,
+					      size_t *ret_n, int *start );
+const byte *parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype,
+				       size_t *ret_n );
 const byte *parse_sig_subpkt2( PKT_signature *sig,
 			       sigsubpkttype_t reqtype, size_t *ret_n );
 
