@@ -640,7 +640,7 @@ parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
     int i, ndata;
     PKT_pubkey_enc *k;
 
-    k = packet->pkt.pubkey_enc = m_alloc(sizeof *packet->pkt.pubkey_enc );
+    k = packet->pkt.pubkey_enc = m_alloc_clear(sizeof *packet->pkt.pubkey_enc);
     if( pktlen < 12 ) {
 	log_error("packet(%d) too short\n", pkttype);
 	goto leave;
@@ -745,7 +745,28 @@ dump_sig_subpkt( int hashed, int type, int critical,
 		      (ulong)buffer_to_u32(buffer+4) );
 	break;
       case SIGSUBPKT_NOTATION:
-	p = "notation data";
+	{
+	    fputs("notation: ", stdout );
+	    if( length < 8 )
+		p = "[too short]";
+	    else if( !(*buffer & 0x80) )
+		p = "[not human readable]";
+	    else {
+		const byte *s = buffer;
+		size_t n1, n2;
+
+		n1 = (s[4] << 8) | s[5];
+		n2 = (s[6] << 8) | s[7];
+		s += 8;
+		if( 8+n1+n2 != length )
+		    p = "[error]";
+		else {
+		    print_string( stdout, s, n1, ')' );
+		    putc( '=', stdout );
+		    print_string( stdout, s+n1, n2, ')' );
+		}
+	    }
+	}
 	break;
       case SIGSUBPKT_PREF_HASH:
 	fputs("pref-hash-algos:", stdout );
@@ -767,7 +788,8 @@ dump_sig_subpkt( int hashed, int type, int critical,
 	p = "primary user id";
 	break;
       case SIGSUBPKT_POLICY:
-	p = "policy URL";
+	fputs("policy: ", stdout );
+	print_string( stdout, buffer, length, ')' );
 	break;
       case SIGSUBPKT_KEY_FLAGS:
 	p = "key flags";
@@ -808,9 +830,14 @@ parse_one_sig_subpkt( const byte *buffer, size_t n, int type )
 	if( n < 8 )
 	    break;
 	return 0;
+      case SIGSUBPKT_NOTATION:
+	if( n < 8 ) /* minimum length needed */
+	    break;
+	return 0;
       case SIGSUBPKT_PREF_SYM:
       case SIGSUBPKT_PREF_HASH:
       case SIGSUBPKT_PREF_COMPR:
+      case SIGSUBPKT_POLICY:
 	return 0;
       case SIGSUBPKT_PRIV_ADD_SIG:
 	/* because we use private data, we check the GNUPG marker */
@@ -824,16 +851,46 @@ parse_one_sig_subpkt( const byte *buffer, size_t n, int type )
     return -3;
 }
 
+
+static int
+can_handle_critical( const byte *buffer, size_t n, int type )
+{
+    switch( type ) {
+      case SIGSUBPKT_NOTATION:
+	if( n >= 8 && (*buffer & 0x80) )
+	    return 1; /* human readable is handled */
+	return 0;
+
+      case SIGSUBPKT_SIG_CREATED:
+      case SIGSUBPKT_SIG_EXPIRE:
+      case SIGSUBPKT_KEY_EXPIRE:
+      case SIGSUBPKT_EXPORTABLE:
+      case SIGSUBPKT_ISSUER:/* issuer key ID */
+      case SIGSUBPKT_PREF_SYM:
+      case SIGSUBPKT_PREF_HASH:
+      case SIGSUBPKT_PREF_COMPR:
+	return 1;
+
+      case SIGSUBPKT_POLICY: /* Is enough to show the policy? */
+      default:
+	return 0;
+    }
+}
+
+
 const byte *
-parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
+enum_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype,
+		 size_t *ret_n, int *start )
 {
     int buflen;
     int type;
     int critical;
     int offset;
     size_t n;
+    int seq = 0;
+    int reqseq = start? *start: 0;
 
-    if( !buffer )
+    if( !buffer || reqseq == -1 )
 	return NULL;
     buflen = (*buffer << 8) | buffer[1];
     buffer += 2;
@@ -864,13 +921,17 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
 	}
 	else
 	    critical = 0;
-	if( reqtype == SIGSUBPKT_TEST_CRITICAL ) {
+	if( !(++seq > reqseq) )
+	    ;
+	else if( reqtype == SIGSUBPKT_TEST_CRITICAL ) {
 	    if( critical ) {
 		if( n-1 > buflen+1 )
 		    goto too_short;
-		if( parse_one_sig_subpkt(buffer+1, n-1, type ) < 0 ) {
+		if( !can_handle_critical(buffer+1, n-1, type ) ) {
 		    log_info(_("subpacket of type %d has critical bit set\n"),
 									type);
+		    if( start )
+			*start = seq;
 		    return NULL; /* this is an error */
 		}
 	    }
@@ -897,6 +958,8 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
 	      default:
 		break;
 	    }
+	    if( start )
+		*start = seq;
 	    return buffer+offset;
 	}
 	buffer += n; buflen -=n;
@@ -904,14 +967,24 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
     if( reqtype == SIGSUBPKT_TEST_CRITICAL )
 	return buffer; /* as value true to indicate that there is no */
 		       /* critical bit we don't understand */
+    if( start )
+	*start = -1;
     return NULL; /* end of packets; not found */
 
   too_short:
     log_error("buffer shorter than subpacket\n");
+    if( start )
+	*start = -1;
     return NULL;
 }
 
 
+const byte *
+parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
+{
+    return enum_sig_subpkt( buffer, reqtype, ret_n, NULL );
+}
+
 const byte *
 parse_sig_subpkt2( PKT_signature *sig, sigsubpkttype_t reqtype, size_t *ret_n )
 {
@@ -1208,7 +1281,7 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 
     if( pkttype == PKT_SECRET_KEY || pkttype == PKT_SECRET_SUBKEY ) {
 	PKT_secret_key *sk = pkt->pkt.secret_key;
-	byte temp[8];
+	byte temp[16];
 
 	if( !npkey ) {
 	    sk->skey[0] = mpi_set_opaque( NULL,
@@ -1297,6 +1370,8 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	     * so a
 	     *	 sk->protect.ivlen = cipher_get_blocksize(sk->protect.algo);
 	     * won't work.  The only solution I see is to hardwire it here.
+	     * NOTE: if you change the ivlen above 16, don't forget to
+	     * enlarge temp.
 	     */
 	    switch( sk->protect.algo ) {
 	      case 7: case 8: case 9: /* reserved for AES */
