@@ -298,7 +298,7 @@ add_keyblock_resource( const char *url, int force, int secret )
 			rc = G10ERR_OPEN_FILE;
 			goto leave;
 		    }
-		    else
+		    else if( !opt.quiet )
 			log_info( _("%s: directory created\n"), filename );
 		    copy_options_file( filename );
 		}
@@ -318,8 +318,20 @@ add_keyblock_resource( const char *url, int force, int secret )
 		rc = G10ERR_OPEN_FILE;
 		goto leave;
 	    }
-	    else
-		log_info(_("%s: keyring created\n"), filename );
+	    else {
+	      #ifndef HAVE_DOSISH_SYSTEM
+		if( secret ) {
+		    if( chmod( filename, S_IRUSR | S_IWUSR ) ) {
+			log_error("%s: chmod failed: %s\n",
+						filename, strerror(errno) );
+			rc = G10ERR_WRITE_FILE;
+			goto leave;
+		    }
+		}
+	      #endif
+		if( !opt.quiet )
+		    log_info(_("%s: keyring created\n"), filename );
+	    }
 	}
       #if HAVE_DOSISH_SYSTEM || 1
 	iobuf_close( iobuf );
@@ -350,6 +362,13 @@ add_keyblock_resource( const char *url, int force, int secret )
 	goto leave;
     }
 
+  #ifndef HAVE_DOSISH_SYSTEM
+  #if 0 /* fixme: check directory permissions and print a warning */
+    if( secret ) {
+    }
+  #endif
+  #endif
+
     /* fixme: avoid duplicate resources */
     resource_table[i].used = 1;
     resource_table[i].secret = !!secret;
@@ -1326,7 +1345,7 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	    unlock_rentry( rentry );
 	    return G10ERR_OPEN_FILE;
 	}
-	else
+	else if( !opt.quiet )
 	    log_info(_("%s: keyring created\n"), rentry->fname );
 
 	kbctx=NULL;
