@@ -39,6 +39,8 @@ struct cmp_help_context_s {
 };
 
 
+static int do_signature_check( PKT_signature *sig, MD_HANDLE digest,
+						      u32 *r_expire );
 static int do_check( PKT_public_key *pk, PKT_signature *sig,
 						MD_HANDLE digest );
 
@@ -50,6 +52,13 @@ static int do_check( PKT_public_key *pk, PKT_signature *sig,
  */
 int
 signature_check( PKT_signature *sig, MD_HANDLE digest )
+{
+    u32 dummy;
+    return do_signature_check( sig, digest, &dummy );
+}
+
+static int
+do_signature_check( PKT_signature *sig, MD_HANDLE digest, u32 *r_expire )
 {
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
     int rc=0;
@@ -57,10 +66,13 @@ signature_check( PKT_signature *sig, MD_HANDLE digest )
     if( is_RSA(sig->pubkey_algo) )
 	write_status(STATUS_RSA_OR_IDEA);
 
+    *r_expire = 0;
     if( get_pubkey( pk, sig->keyid ) )
 	rc = G10ERR_NO_PUBKEY;
-    else
+    else {
+	*r_expire = pk->expiredate;
 	rc = do_check( pk, sig, digest );
+    }
 
     free_public_key( pk );
 
@@ -397,6 +409,13 @@ hash_uid_node( KBNODE unode, MD_HANDLE md, PKT_signature *sig )
  */
 int
 check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
+{
+    u32 dummy;
+    return check_key_signature2(root, node, is_selfsig, &dummy );
+}
+
+int
+check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 {
     MD_HANDLE md;
     PKT_public_key *pk;
@@ -406,12 +425,18 @@ check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 
     if( is_selfsig )
 	*is_selfsig = 0;
+    *r_expire = 0;
     assert( node->pkt->pkttype == PKT_SIGNATURE );
     assert( root->pkt->pkttype == PKT_PUBLIC_KEY );
 
     pk = root->pkt->pkt.public_key;
     sig = node->pkt->pkt.signature;
     algo = sig->digest_algo;
+
+    if( sig->flags.checked )
+	log_debug("check_key_signature: already checked: %s\n",
+		      sig->flags.valid? "good":"bad" );
+
     if( (rc=check_digest_algo(algo)) )
 	return rc;
 
@@ -474,7 +499,7 @@ check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 		rc = do_check( pk, sig, md );
 	    }
 	    else
-		rc = signature_check( sig, md );
+		rc = do_signature_check( sig, md, r_expire );
 	    md_close(md);
 	}
 	else {
