@@ -125,7 +125,7 @@ asctimestamp( u32 stamp )
  * Print a string to FP, but filter all control characters out.
  */
 void
-print_string( FILE *fp, byte *p, size_t n, int delim )
+print_string( FILE *fp, const byte *p, size_t n, int delim )
 {
     for( ; n; n--, p++ )
 	if( iscntrl( *p ) || *p == delim ) {
@@ -164,3 +164,26 @@ answer_is_yes( const char *s )
 }
 
 
+/****************
+ * Return 1 for yes, -1 for quit, or 0 for no
+ */
+int
+answer_is_yes_no_quit( const char *s )
+{
+    char *long_yes = _("yes");
+    char *long_quit = _("quit");
+    char *short_yes = _("yY");
+    char *short_quit = _("qQ");
+
+    if( !stricmp(s, long_yes ) )
+	return 1;
+    if( !stricmp(s, long_quit ) )
+	return -1;
+    if( strchr( short_yes, *s ) && !s[1] )
+	return 1;
+    if( strchr( short_quit, *s ) && !s[1] )
+	return -1;
+    return 0;
+}
+
+
